package com.seeyon.cip.spi.user.auth.thirdauth.example;

import com.seeyon.cip.spi.user.auth.thirdauth.constants.ThirdauthSsoAuthConstants;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.sso.CtpUserSsoAuthProviderService;
import com.seeyon.ctp.user.dto.CtpUserSpiLoginUserInfoDto;
import com.seeyon.ctp.user.dto.CtpUserSpiThirdTokenDto;
import com.seeyon.ctp.user.exception.CtpUserSpiSsoException;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;

/**
 * <pre>
 * 浏览器 cookie 中需要保存三方认证中心的 token
 * </pre>
 */
@Slf4j
@CtpUserChannelRouter(ThirdauthSsoAuthConstants.THIRD_TYPE) // 注： 注解必有要有
public class C_TokenSaveProviderService implements CtpUserSsoAuthProviderService {

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getRequestParaKey() {
        return "code";
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getSsoLoginUrl(HttpServletRequest request, String encodeRedirectUrl) {
        // 三方登录地址： http://simple.sso.com?redirectUrl=xxx
        // ctp-user 会拼装好三方登录成功后的重定向地址， 并通过 url encode 编码，默认情况下只需要追加到 redirectUrl 后面即可
        String thirdLoginUrl = "http://simple.sso.com?redirectUrl=" + encodeRedirectUrl;
        log.info("跳转到三方认证中心地址:{}", thirdLoginUrl);
        return thirdLoginUrl;
    }

    /**
     * 保存到浏览器 cookie 里三方认证中心的 token的 key，需要中横杠分隔，且不能为空，如果为空则不保存
     */
    @Override
    public String get4aTokenKey() {
        return "access-4a-token";
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     * web 端登录三方认证中心成功后， 获取用户信息方法
     * 注： 需要增加三方 token 的返回信息， 否则无法保存到 cookie 中
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfo(HttpServletRequest request, String encodeRedirectUrl) throws CtpUserSpiSsoException {
        // 通过接口获取三方用户信息过程中， 获取到的三方 token
        String thirdToken = "test_third_token";
        // 保存三方 token 信息，和用户信息一起返回
        CtpUserSpiThirdTokenDto ctpUserSpiThirdTokenDto = CtpUserSpiThirdTokenDto.builder()
                .thirdToken(thirdToken)
                .expired(3600L) // 过期时间
                .build();
        return CtpUserSpiLoginUserInfoDto.builder()
                .thirdLoginName("zhangsan") // 用户信息
                .ctpUserSpiThirdTokenDto(ctpUserSpiThirdTokenDto) // 三方 token 信息
                .build();
    }
}