package com.seeyon.cip.spi.user.auth.thirdauth.example;

import com.seeyon.cip.spi.user.auth.thirdauth.constants.ThirdauthSsoAuthConstants;
import com.seeyon.ctp.user.annotation.CtpUserChannelRouter;
import com.seeyon.ctp.user.api.sso.CtpUserSsoAuthProviderService;
import com.seeyon.ctp.user.dto.CtpUserSpiLoginUserInfoDto;
import com.seeyon.ctp.user.exception.CtpUserSpiSsoException;
import lombok.extern.slf4j.Slf4j;

import javax.servlet.http.HttpServletRequest;
import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;

/**
 * <pre>
 * v8 通过三方认证中心登录后， 需要登出的情况
 * </pre>
 */
@Slf4j
@CtpUserChannelRouter(ThirdauthSsoAuthConstants.THIRD_TYPE) // 注： 注解必有要有
public class D_LogOutProviderService implements CtpUserSsoAuthProviderService {

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getRequestParaKey() {
        return "code";
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     */
    @Override
    public String getSsoLoginUrl(HttpServletRequest request, String encodeRedirectUrl) {
        // 三方登录地址： http://simple.sso.com?redirectUrl=xxx
        // ctp-user 会拼装好三方登录成功后的重定向地址， 并通过 url encode 编码，默认情况下只需要追加到 redirectUrl 后面即可
        String thirdLoginUrl = "http://simple.sso.com?redirectUrl=" + encodeRedirectUrl;
        log.info("跳转到三方认证中心地址:{}", thirdLoginUrl);
        return thirdLoginUrl;
    }

    /**
     * 同 A_SimpleAuthProviderService 配置
     * web 端登录三方认证中心成功后， 获取用户信息方法
     * 注： 需要增加三方 token 的返回信息， 否则无法保存到 cookie 中
     */
    @Override
    public CtpUserSpiLoginUserInfoDto getUserLoginInfo(HttpServletRequest request, String encodeRedirectUrl) throws CtpUserSpiSsoException {
        return CtpUserSpiLoginUserInfoDto.builder()
                .thirdLoginName("zhangsan") // 用户信息
                .build();
    }

    /**
     * <pre>
     * 注： 返回值地址需要 url encode
     * 三方登出重定向地址，该方法有实现时，v8 页面中会出现登出地址， 否则不会出现登出地址
     * 该地址为三方认证中心的登录地址 + 重定向到 v8 的地址。
     * 如 三方认证中心的登录地址为： https://third.logout.com?redirectUrl=https%3A%2F%2Fdev.seeyonv8.com%2Fmain%2Fportal
     * 方法返回值则为： https%3A%2F%2Fthird.logout.com%3FredirectUrl%3Dhttps%253A%252F%252Fdev.seeyonv8.com%252Fmain%252Fportal
     * </pre>
     */
    @Override
    public String getEncodeLogoutRedirectUrl() {
        String v8redirectUrl = null;
        try {
            v8redirectUrl = URLEncoder.encode("https://dev.seeyonv8.com/main/portal", "UTF-8");
            String thirdLogoutUrl = "https://third.logout.com?redirectUrl=" + v8redirectUrl;
            String encodeLogoutUrl = URLEncoder.encode(thirdLogoutUrl, "UTF-8");
            return encodeLogoutUrl;
        } catch (UnsupportedEncodingException e) {
            log.error("encodeLogoutRedirectUrl error", e);
            return null;
        }
    }
}